@@ -1,3 +1,4 @@
+#define USE_FC_LEN_T
 #include <string>
 #include "util.h"
 #include "rpg.h"
@@ -12,6 +13,9 @@
 #include <R_ext/Linpack.h>
 #include <R_ext/Lapack.h>
 #include <R_ext/BLAS.h>
+#ifndef FCONE
+# define FCONE
+#endif
 
 extern "C" {
   SEXP msPGOcc(SEXP y_r, SEXP X_r, SEXP Xp_r, SEXP XRE_r, SEXP XpRE_r, 
@@ -253,38 +257,38 @@ extern "C" {
     double *ySum = (double *) R_alloc(JN, sizeof(double)); zeros(ySum, JN);
 
     // For normal priors
-    F77_NAME(dpotrf)(lower, &pOcc, SigmaBetaCommInv, &pOcc, &info); 
+    F77_NAME(dpotrf)(lower, &pOcc, SigmaBetaCommInv, &pOcc, &info FCONE); 
     if(info != 0){error("c++ error: dpotrf SigmaBetaCommInv failed\n");}
-    F77_NAME(dpotri)(lower, &pOcc, SigmaBetaCommInv, &pOcc, &info); 
+    F77_NAME(dpotri)(lower, &pOcc, SigmaBetaCommInv, &pOcc, &info FCONE); 
     if(info != 0){error("c++ error: dpotri SigmaBetaCommInv failed\n");}
     double *SigmaBetaCommInvMuBeta = (double *) R_alloc(pOcc, sizeof(double)); 
     F77_NAME(dsymv)(lower, &pOcc, &one, SigmaBetaCommInv, &pOcc, muBetaComm, &inc, &zero, 
-        	    SigmaBetaCommInvMuBeta, &inc);
+        	    SigmaBetaCommInvMuBeta, &inc FCONE);
     // Detection regression coefficient priors. 
-    F77_NAME(dpotrf)(lower, &pDet, SigmaAlphaCommInv, &pDet, &info); 
+    F77_NAME(dpotrf)(lower, &pDet, SigmaAlphaCommInv, &pDet, &info FCONE); 
     if(info != 0){error("c++ error: dpotrf SigmaAlphaCommInv failed\n");}
-    F77_NAME(dpotri)(lower, &pDet, SigmaAlphaCommInv, &pDet, &info); 
+    F77_NAME(dpotri)(lower, &pDet, SigmaAlphaCommInv, &pDet, &info FCONE); 
     if(info != 0){error("c++ error: dpotri SigmaAlphaCommInv failed\n");}
     double *SigmaAlphaCommInvMuAlpha = (double *) R_alloc(pDet, sizeof(double)); 
     F77_NAME(dsymv)(lower, &pDet, &one, SigmaAlphaCommInv, &pDet, muAlphaComm, &inc, &zero, 
-                   SigmaAlphaCommInvMuAlpha, &inc);
+                   SigmaAlphaCommInvMuAlpha, &inc FCONE);
     // Put community level variances in a pOcc x POcc matrix.
     double *TauBetaInv = (double *) R_alloc(ppOcc, sizeof(double)); zeros(TauBetaInv, ppOcc); 
     for (i = 0; i < pOcc; i++) {
       TauBetaInv[i * pOcc + i] = tauSqBeta[i]; 
     } // i
-    F77_NAME(dpotrf)(lower, &pOcc, TauBetaInv, &pOcc, &info); 
+    F77_NAME(dpotrf)(lower, &pOcc, TauBetaInv, &pOcc, &info FCONE); 
     if(info != 0){error("c++ error: dpotrf TauBetaInv failed\n");}
-    F77_NAME(dpotri)(lower, &pOcc, TauBetaInv, &pOcc, &info); 
+    F77_NAME(dpotri)(lower, &pOcc, TauBetaInv, &pOcc, &info FCONE); 
     if(info != 0){error("c++ error: dpotri TauBetaInv failed\n");}
     // Put community level variances in a pDet x pDet matrix. 
     double *TauAlphaInv = (double *) R_alloc(ppDet, sizeof(double)); zeros(TauAlphaInv, ppDet); 
     for (i = 0; i < pDet; i++) {
       TauAlphaInv[i * pDet + i] = tauSqAlpha[i]; 
     } // i
-    F77_NAME(dpotrf)(lower, &pDet, TauAlphaInv, &pDet, &info); 
+    F77_NAME(dpotrf)(lower, &pDet, TauAlphaInv, &pDet, &info FCONE); 
     if(info != 0){error("c++ error: dpotrf TauAlphaInv failed\n");}
-    F77_NAME(dpotri)(lower, &pDet, TauAlphaInv, &pDet, &info); 
+    F77_NAME(dpotri)(lower, &pDet, TauAlphaInv, &pDet, &info FCONE); 
     if(info != 0){error("c++ error: dpotri TauAlphaInv failed\n");}
 
     /**********************************************************************
@@ -334,7 +338,7 @@ extern "C" {
        *******************************/
       zeros(tmp_pOcc, pOcc); 
       for (i = 0; i < N; i++) {
-        F77_NAME(dgemv)(ytran, &pOcc, &pOcc, &one, TauBetaInv, &pOcc, &beta[i], &N, &one, tmp_pOcc, &inc); 
+        F77_NAME(dgemv)(ytran, &pOcc, &pOcc, &one, TauBetaInv, &pOcc, &beta[i], &N, &one, tmp_pOcc, &inc FCONE); 
       } // i
       for (q = 0; q < pOcc; q++) {
         tmp_pOcc[q] += SigmaBetaCommInvMuBeta[q];  
@@ -346,12 +350,12 @@ extern "C" {
       for (q = 0; q < ppOcc; q++) {
         tmp_ppOcc[q] = SigmaBetaCommInv[q] + N * TauBetaInv[q]; 
       }
-      F77_NAME(dpotrf)(lower, &pOcc, tmp_ppOcc, &pOcc, &info); 
+      F77_NAME(dpotrf)(lower, &pOcc, tmp_ppOcc, &pOcc, &info FCONE); 
       if(info != 0){error("c++ error: dpotrf ABetaComm failed\n");}
-      F77_NAME(dpotri)(lower, &pOcc, tmp_ppOcc, &pOcc, &info); 
+      F77_NAME(dpotri)(lower, &pOcc, tmp_ppOcc, &pOcc, &info FCONE); 
       if(info != 0){error("c++ error: dpotri ABetaComm failed\n");}
-      F77_NAME(dsymv)(lower, &pOcc, &one, tmp_ppOcc, &pOcc, tmp_pOcc, &inc, &zero, tmp_pOcc2, &inc);
-      F77_NAME(dpotrf)(lower, &pOcc, tmp_ppOcc, &pOcc, &info); 
+      F77_NAME(dsymv)(lower, &pOcc, &one, tmp_ppOcc, &pOcc, tmp_pOcc, &inc, &zero, tmp_pOcc2, &inc FCONE);
+      F77_NAME(dpotrf)(lower, &pOcc, tmp_ppOcc, &pOcc, &info FCONE); 
       if(info != 0){error("c++ error: dpotrf ABetaComm failed\n");}
       mvrnorm(betaComm, tmp_pOcc2, tmp_ppOcc, pOcc);
       /********************************************************************
@@ -362,7 +366,7 @@ extern "C" {
        *******************************/
        zeros(tmp_pDet, pDet); 
        for (i = 0; i < N; i++) {
-         F77_NAME(dgemv)(ytran, &pDet, &pDet, &one, TauAlphaInv, &pDet, &alpha[i], &N, &one, tmp_pDet, &inc); 
+         F77_NAME(dgemv)(ytran, &pDet, &pDet, &one, TauAlphaInv, &pDet, &alpha[i], &N, &one, tmp_pDet, &inc FCONE); 
        } // i
        for (q = 0; q < pDet; q++) {
          tmp_pDet[q] += SigmaAlphaCommInvMuAlpha[q];  
@@ -373,12 +377,12 @@ extern "C" {
       for (q = 0; q < ppDet; q++) {
         tmp_ppDet[q] = SigmaAlphaCommInv[q] + N * TauAlphaInv[q]; 
       }
-      F77_NAME(dpotrf)(lower, &pDet, tmp_ppDet, &pDet, &info); 
+      F77_NAME(dpotrf)(lower, &pDet, tmp_ppDet, &pDet, &info FCONE); 
       if(info != 0){error("c++ error: dpotrf AAlphaComm failed\n");}
-      F77_NAME(dpotri)(lower, &pDet, tmp_ppDet, &pDet, &info); 
+      F77_NAME(dpotri)(lower, &pDet, tmp_ppDet, &pDet, &info FCONE); 
       if(info != 0){error("c++ error: dpotri AAlphaComm failed\n");}
-      F77_NAME(dsymv)(lower, &pDet, &one, tmp_ppDet, &pDet, tmp_pDet, &inc, &zero, tmp_pDet2, &inc);
-      F77_NAME(dpotrf)(lower, &pDet, tmp_ppDet, &pDet, &info); 
+      F77_NAME(dsymv)(lower, &pDet, &one, tmp_ppDet, &pDet, tmp_pDet, &inc, &zero, tmp_pDet2, &inc FCONE);
+      F77_NAME(dpotrf)(lower, &pDet, tmp_ppDet, &pDet, &info FCONE); 
       if(info != 0){error("c++ error: dpotrf AAlphaComm failed\n");}
       mvrnorm(alphaComm, tmp_pDet2, tmp_ppDet, pDet);
 
@@ -396,9 +400,9 @@ extern "C" {
       for (q = 0; q < pOcc; q++) {
         TauBetaInv[q * pOcc + q] = tauSqBeta[q]; 
       } // q
-      F77_NAME(dpotrf)(lower, &pOcc, TauBetaInv, &pOcc, &info); 
+      F77_NAME(dpotrf)(lower, &pOcc, TauBetaInv, &pOcc, &info FCONE); 
       if(info != 0){error("c++ error: dpotrf TauBetaInv failed\n");}
-      F77_NAME(dpotri)(lower, &pOcc, TauBetaInv, &pOcc, &info); 
+      F77_NAME(dpotri)(lower, &pOcc, TauBetaInv, &pOcc, &info FCONE); 
       if(info != 0){error("c++ error: dpotri TauBetaInv failed\n");}
       /********************************************************************
        Update Community Detection Variance Parameter
@@ -414,9 +418,9 @@ extern "C" {
       for (q = 0; q < pDet; q++) {
         TauAlphaInv[q * pDet + q] = tauSqAlpha[q]; 
       } // q
-      F77_NAME(dpotrf)(lower, &pDet, TauAlphaInv, &pDet, &info); 
+      F77_NAME(dpotrf)(lower, &pDet, TauAlphaInv, &pDet, &info FCONE); 
       if(info != 0){error("c++ error: dpotrf TauAlphaInv failed\n");}
-      F77_NAME(dpotri)(lower, &pDet, TauAlphaInv, &pDet, &info); 
+      F77_NAME(dpotri)(lower, &pDet, TauAlphaInv, &pDet, &info FCONE); 
       if(info != 0){error("c++ error: dpotri TauAlphaInv failed\n");}
 
       /********************************************************************
@@ -480,9 +484,9 @@ extern "C" {
         /********************************
          * Compute b.beta
          *******************************/
-        F77_NAME(dgemv)(ytran, &J, &pOcc, &one, X, &J, tmp_J1, &inc, &zero, tmp_pOcc, &inc); 	 
+        F77_NAME(dgemv)(ytran, &J, &pOcc, &one, X, &J, tmp_J1, &inc, &zero, tmp_pOcc, &inc FCONE); 	 
         // TauBetaInv %*% betaComm + tmp_pOcc = tmp_pOcc
-        F77_NAME(dgemv)(ntran, &pOcc, &pOcc, &one, TauBetaInv, &pOcc, betaComm, &inc, &one, tmp_pOcc, &inc); 
+        F77_NAME(dgemv)(ntran, &pOcc, &pOcc, &one, TauBetaInv, &pOcc, betaComm, &inc, &one, tmp_pOcc, &inc FCONE); 
 
         /********************************
          * Compute A.beta
@@ -492,16 +496,16 @@ extern "C" {
             tmp_JpOcc[q*J+j] = X[q*J+j]*omegaOcc[j];
           }
         }
-        F77_NAME(dgemm)(ytran, ntran, &pOcc, &pOcc, &J, &one, X, &J, tmp_JpOcc, &J, &zero, tmp_ppOcc, &pOcc);
+        F77_NAME(dgemm)(ytran, ntran, &pOcc, &pOcc, &J, &one, X, &J, tmp_JpOcc, &J, &zero, tmp_ppOcc, &pOcc FCONE FCONE);
         for (q = 0; q < ppOcc; q++) {
           tmp_ppOcc[q] += TauBetaInv[q]; 
         } // q
-        F77_NAME(dpotrf)(lower, &pOcc, tmp_ppOcc, &pOcc, &info); 
+        F77_NAME(dpotrf)(lower, &pOcc, tmp_ppOcc, &pOcc, &info FCONE); 
         if(info != 0){error("c++ error: dpotrf ABeta failed\n");}
-        F77_NAME(dpotri)(lower, &pOcc, tmp_ppOcc, &pOcc, &info); 
+        F77_NAME(dpotri)(lower, &pOcc, tmp_ppOcc, &pOcc, &info FCONE); 
         if(info != 0){error("c++ error: dpotri ABeta failed\n");}
-        F77_NAME(dsymv)(lower, &pOcc, &one, tmp_ppOcc, &pOcc, tmp_pOcc, &inc, &zero, tmp_pOcc2, &inc);
-        F77_NAME(dpotrf)(lower, &pOcc, tmp_ppOcc, &pOcc, &info); 
+        F77_NAME(dsymv)(lower, &pOcc, &one, tmp_ppOcc, &pOcc, tmp_pOcc, &inc, &zero, tmp_pOcc2, &inc FCONE);
+        F77_NAME(dpotrf)(lower, &pOcc, tmp_ppOcc, &pOcc, &info FCONE); 
         if(info != 0){error("c++ error: dpotrf here failed\n");}
         mvrnorm(tmp_beta, tmp_pOcc2, tmp_ppOcc, pOcc);
         for (q = 0; q < pOcc; q++) {
@@ -530,8 +534,8 @@ extern "C" {
           } // r
         }
         
-        F77_NAME(dgemv)(ytran, &nObs, &pDet, &one, Xp, &nObs, tmp_nObs, &inc, &zero, tmp_pDet, &inc); 	  
-        F77_NAME(dgemv)(ntran, &pDet, &pDet, &one, TauAlphaInv, &pDet, alphaComm, &inc, &one, tmp_pDet, &inc); 
+        F77_NAME(dgemv)(ytran, &nObs, &pDet, &one, Xp, &nObs, tmp_nObs, &inc, &zero, tmp_pDet, &inc FCONE); 	  
+        F77_NAME(dgemv)(ntran, &pDet, &pDet, &one, TauAlphaInv, &pDet, alphaComm, &inc, &one, tmp_pDet, &inc FCONE); 
         /********************************
          * Compute A.alpha
          * *****************************/
@@ -541,17 +545,17 @@ extern "C" {
           } // i
         } // j
 
-        F77_NAME(dgemm)(ytran, ntran, &pDet, &pDet, &nObs, &one, Xp, &nObs, tmp_nObspDet, &nObs, &zero, tmp_ppDet, &pDet);
+        F77_NAME(dgemm)(ytran, ntran, &pDet, &pDet, &nObs, &one, Xp, &nObs, tmp_nObspDet, &nObs, &zero, tmp_ppDet, &pDet FCONE FCONE);
 
         for (q = 0; q < ppDet; q++) {
           tmp_ppDet[q] += TauAlphaInv[q]; 
         } // q
-        F77_NAME(dpotrf)(lower, &pDet, tmp_ppDet, &pDet, &info); 
+        F77_NAME(dpotrf)(lower, &pDet, tmp_ppDet, &pDet, &info FCONE); 
         if(info != 0){error("c++ error: dpotrf A.alpha failed\n");}
-        F77_NAME(dpotri)(lower, &pDet, tmp_ppDet, &pDet, &info); 
+        F77_NAME(dpotri)(lower, &pDet, tmp_ppDet, &pDet, &info FCONE); 
         if(info != 0){error("c++ error: dpotri A.alpha failed\n");}
-        F77_NAME(dsymv)(lower, &pDet, &one, tmp_ppDet, &pDet, tmp_pDet, &inc, &zero, tmp_pDet2, &inc);
-        F77_NAME(dpotrf)(lower, &pDet, tmp_ppDet, &pDet, &info); 
+        F77_NAME(dsymv)(lower, &pDet, &one, tmp_ppDet, &pDet, tmp_pDet, &inc, &zero, tmp_pDet2, &inc FCONE);
+        F77_NAME(dpotrf)(lower, &pDet, tmp_ppDet, &pDet, &info FCONE); 
         if(info != 0){error("c++ error: dpotrf here failed\n");}
         mvrnorm(tmp_alpha, tmp_pDet2, tmp_ppDet, pDet);
         for (q = 0; q < pDet; q++) {
